indexing
  description: "A basic savings account"
  author: "Gunnar Gotshalks"
  date: "$2004 January 31$"
  revision: "$1.0.0$"

class SAVINGS_ACCOUNT 

-- A savings account that can be created at one time but no transactions
-- can be made until the account is opened with an initial balance.  At which
-- point deposits and withdrawals can be made.  Eventually the account can
-- be closed.  After closing no further transactions can be made but the
-- balance can be inspected.

create make

--========================
feature {NONE} -- creation

make is
    -- create an unopened savings account.  Transactions cannot be done until
    -- the account is opened.
  require true
  local
  	a_transaction : TRANSACTION
  	now : DATE
  do
    status := acct_not_open
    create transaction_list.make
    create now.make_now
    bcd.generate_next_account_number
    number := bcd.last_account_number
    create a_transaction.make(Current, bcd.trans_create, 0, now)
    transaction_list.extend(a_transaction)
  ensure
    status = acct_not_open
    transaction_list.count = 1
    transaction_list.last.account = current
    transaction_list.last.type = bcd.trans_create 
    bcd /= void
  end

--========================
feature -- Access
  
  balance : REAL    -- Running total of deposits & withdrawals beginning
                    -- with initial_depost

  status  : REAL    -- State of the account is one of the following.
    acct_not_open, acct_open, acct_closed : INTEGER is unique

  transaction_list : LINKED_LIST[TRANSACTION]

  number : INTEGER  -- The unique account number
  
--========================
feature -- Element change

open(initial_deposit : REAL) is
    -- Open a savings account with initial balance of 'initial_deposit'
    -- After opening it transactions on the account can be done.
  require
    status = acct_not_open
    initial_deposit >= 0
  local
  	a_transaction : TRANSACTION
  	now : DATE
  do
    balance := initial_deposit ; status := acct_open
    create now.make_now
    create a_transaction.make(Current, bcd.trans_open, initial_deposit, now)
    transaction_list.extend(a_transaction)
  ensure
    balance = initial_deposit and status = acct_open
    transaction_list.count = old transaction_list.count + 1
    transaction_list.last.account = Current
    transaction_list.last.type = bcd.trans_open
    transaction_list.last.amount = initial_deposit
  end

close is
  -- Close an opened savings account.  No further transactions can be done
  -- but the balance can be inspected.
  require  status = acct_open
  local
  	a_transaction : TRANSACTION
  	now : DATE
  do
    status := acct_closed
    create now.make_now
    create a_transaction.make(Current, bcd.trans_close, 0, now)
    transaction_list.extend(a_transaction)
  ensure
    status = acct_closed
    transaction_list.count = old transaction_list.count + 1
    transaction_list.last.account = Current
    transaction_list.last.type = bcd.trans_close 
  end
    
deposit (amount : REAL) is
    -- Add the amount to the account balance
  require
    status = acct_open
    amount_positive: amount> 0
  local
  	a_transaction : TRANSACTION
  	now : DATE
  do
    balance := balance + amount
    create now.make_now
    create a_transaction.make(Current, bcd.trans_deposit, amount, now)
    transaction_list.extend(a_transaction)
  ensure
    balance = old balance + amount
    status = old status
    transaction_list.count = old transaction_list.count + 1
    transaction_list.last.account = Current
    transaction_list.last.type = bcd.trans_deposit
    transaction_list.last.amount = amount
  end

withdraw (amount : REAL) is
    -- Subtract the amount from the account balance
  require
    status = acct_open
    amount > 0
    amount <= balance
  local
  	a_transaction : TRANSACTION
  	now : DATE
  do
    balance := balance - amount
    create now.make_now
    create a_transaction.make(Current, bcd.trans_withdraw, amount, now)
    transaction_list.extend(a_transaction)
  ensure
    balance = old balance - amount
    status = old status
    transaction_list.count = old transaction_list.count + 1
    transaction_list.last.account = Current
    transaction_list.last.type = bcd.trans_withdraw
    transaction_list.last.amount = amount
  end
  
--========================
feature {NONE} -- Implementation
		
bcd : BANK_COMMON_DATA is
    -- access to common bank data
  local
	da: BANK_DATA_ACCESS
  once
	da := create{BANK_DATA_ACCESS}
	Result := da.data
  end

invariant
  balance >= 0

end -- class SAVINGS_ACCOUNT
